<?php
namespace phpCtrl;

// Desc: utility/tool shared functions
class C_Utility{

    // Desc: Utility function to add slashes - add slashes only the magic_quotes_gpc is set to off
    // It is strongly recommended that to turn off the magic quotes in the configuration setting
    public static function add_slashes($str){
        //if (get_magic_quotes_gpc() == 1) {
        //    return ($str);
        //}else{ 
            return (addslashes($str));
        //}
    }

     // Indents JSON string to be more readable
     public static function indent_json($json) {
        $result    = '';
        $pos       = 0;
        $strLen    = strlen($json);
        $indentStr = '  ';
        $newLine   = "\n";

        for($i = 0; $i <= $strLen; $i++) {

            // Grab the next character in the string
            $char = substr($json, $i, 1);

            // If this character is the end of an element,
            // output a new line and indent the next line
            if($char == '}' || $char == ']') {
                $result .= $newLine;
                $pos --;
                for ($j=0; $j<$pos; $j++) {
                    $result .= $indentStr;
                }
            }

            // Add the character to the result string
            $result .= $char;

            // If the last character was the beginning of an element, 
            // output a new line and indent the next line
            #bug: it adds newline wronly when the value is comma(,). Removed for now
            //if ($char == ',' || $char == '{' || $char == '[') {
            if ($char == '{' || $char == '[') {
                $result .= $newLine;
                if ($char == '{' || $char == '[') {
                    $pos ++;
                }
                for ($j = 0; $j < $pos; $j++) {
                    $result .= $indentStr;
                }
            }
        }

        return $result;
    }

    // convert boolean to literal string used by jqgrid script;
    public static function literalBool($boolValue){
        return ($boolValue)?'true':'false';
    }

    // generate grid rowid from single or composite PK
    public static function gen_rowids($arr=array(), $keys = array()){
        $rowids = '';
        if(is_array($keys)) { 

            // set all fields to the same case (UPPER). SELECT * will return whatever case is used in db schema
            if (PHPGRID_DB_TYPE == 'db2') {
                $keys = array_map('strtoupper', $keys);
                $arr = array_change_key_case($arr, CASE_UPPER);
            }

            foreach($keys as $key=>$val){
                $val = str_replace('`', '', $val);
                $rowids .= $arr[$val] .PK_DELIMITER;
            }
        }

        $rowids = substr($rowids, 0, -3);   // remove the last PK_DELIMITER

        return $rowids;
    }

    public static function is_debug(){
        return defined('DEBUG')?DEBUG:false;
    }

    // simple encryption decription using AES-256-CBC method
    // credit: https://nazmulahsan.me/simple-two-way-function-encrypt-decrypt-string/
    public static function simple_crypt( $data, $secret_key, $secret_iv, $action = 'e' ) {
        $output = false;
        $encrypt_method = "AES-256-CBC";
        $key = hash( 'sha256', $secret_key );
        $iv = substr( hash( 'sha256', $secret_iv ), 0, 16 );

        if( $action == 'e' ) {
            $output = base64_encode( openssl_encrypt( $data, $encrypt_method, $key, 0, $iv ) );
        }
        else if( $action == 'd' ){
            $output = openssl_decrypt( base64_decode( $data ), $encrypt_method, $key, 0, $iv );
        }

        return $output;
    }

    //  "white list" where we only list allowed values. Can be used for preventing SQL injection
    // ref: https://phpdelusions.net/pdo_examples/order_by
    public static function white_list(&$value, $allowed, $message) {

        // drag and drop grouping has $value like 'status asc, 1', only need the first word

        $value = explode(',', $value)[0];
        $value = str_replace(' asc', '', $value);
        $value = str_replace(' desc', '', $value);

        if ($value === null) {
            return $allowed[0];
        }
        $key = array_search($value, $allowed, true);

        // 1 is the dfault value for sort  name used after ORDERBY
        // but remember everything thru $_GET is a string where 
        if ($key === false && $value !== '1') {
            throw new \InvalidArgumentException($message);
        } else {
            return $value;
        }
    }

}